@@ -50,39 +50,44 @@ int dladdr(void *address, Dl_info *dl)
 
 extern gchar *_gst_progname;
 
+GStaticPrivate _gst_debug_cothread_index = G_STATIC_PRIVATE_INIT;
+
 
 /***** Categories and colorization *****/
+/* be careful with these, make them match the enum */
 static gchar *_gst_info_category_strings[] = {
-  "GST_INIT",
-  "COTHREADS",
-  "COTHREAD_SWITCH",
-  "AUTOPLUG",
-  "AUTOPLUG_ATTEMPT",
-  "PARENTAGE",
-  "STATES",
-  "PLANNING",
-  "SCHEDULING",
-  "DATAFLOW",
-  "BUFFER",
-  "CAPS",
-  "CLOCK",
-  "ELEMENT_PADS",
-  "ELEMENTFACTORY",
-  "PADS",
-  "PIPELINE",
-  "PLUGIN_LOADING",
-  "PLUGIN_ERRORS",
-  "PLUGIN_INFO",
-  "PROPERTIES",
-  "THREAD",
-  "TYPES",
-  "XML",
-  "NEGOTIATION",
-  "REFCOUNTING",
-  "EVENT",
-  "PARAMS",
-
-  [30] = "CALL_TRACE",
+  /* [GST_CAT_GST_INIT]		= */ "GST_INIT",
+  /* [GST_CAT_COTHREADS]	= */ "COTHREADS",
+  /* [GST_CAT_COTHREAD_SWITCH]	= */ "COTHREAD_SWITCH",
+  /* [GST_CAT_AUTOPLUG]		= */ "AUTOPLUG",
+  /* [GST_CAT_AUTOPLUG_ATTEMPT]	= */ "AUTOPLUG_ATTEMPT",
+  /* [GST_CAT_PARENTAGE]	= */ "PARENTAGE",
+  /* [GST_CAT_STATES]		= */ "STATES",
+  /* [GST_CAT_PLANNING]		= */ "PLANNING",
+  /* [GST_CAT_SCHEDULING]	= */ "SCHEDULING",
+  /* [GST_CAT_DATAFLOW]		= */ "DATAFLOW",
+  /* [GST_CAT_BUFFER]		= */ "BUFFER",
+  /* [GST_CAT_CAPS]		= */ "CAPS",
+  /* [GST_CAT_CLOCK]		= */ "CLOCK",
+  /* [GST_CAT_ELEMENT_PADS]	= */ "ELEMENT_PADS",
+  /* [GST_CAT_ELEMENT_FACTORY]	= */ "ELEMENTFACTORY",
+  /* [GST_CAT_PADS]		= */ "PADS",
+  /* [GST_CAT_PIPELINE]		= */ "PIPELINE",
+  /* [GST_CAT_PLUGIN_LOADING]	= */ "PLUGIN_LOADING",
+  /* [GST_CAT_PLUGIN_ERRORS]	= */ "PLUGIN_ERRORS",
+  /* [GST_CAT_PLUGIN_INFO]	= */ "PLUGIN_INFO",
+  /* [GST_CAT_PROPERTIES]	= */ "PROPERTIES",
+  /* [GST_CAT_THREAD]		= */ "THREAD",
+  /* [GST_CAT_TYPES]		= */ "TYPES",
+  /* [GST_CAT_XML]		= */ "XML",
+  /* [GST_CAT_NEGOTIATION]	= */ "NEGOTIATION",
+  /* [GST_CAT_REFCOUNTING]	= */ "REFCOUNTING",
+  /* [GST_CAT_EVENT]		= */ "EVENT",
+  /* [GST_CAT_PARAMS]		= */ "PARAMS",
+  "",
+  "",
+  /* [GST_CAT_CALL_TRACE]	= */ "CALL_TRACE",
+  /* [31]			= */ 
 };
 
 /**
@@ -110,38 +115,40 @@ gst_get_category_name (gint category) {
  * Background color codes:
  * 40=black 41=red 42=green 43=yellow 44=blue 45=magenta 46=cyan 47=white
  */
+/* be careful with these, make them match the enum */
 const gchar *_gst_category_colors[32] = {
-  [GST_CAT_GST_INIT]		= "07;37",
-  [GST_CAT_COTHREADS]		= "00;32",
-  [GST_CAT_COTHREAD_SWITCH]	= "00;37;42",
-  [GST_CAT_AUTOPLUG]		= "00;34",
-  [GST_CAT_AUTOPLUG_ATTEMPT]	= "00;36;44",
-  [GST_CAT_PARENTAGE]		= "01;37;41",		/* !! */
-  [GST_CAT_STATES]		= "00;31",
-  [GST_CAT_PLANNING]		= "07;35",
-  [GST_CAT_SCHEDULING]		= "00;35",
-  [GST_CAT_DATAFLOW]		= "00;32",
-  [GST_CAT_BUFFER]		= "00;32",
-  [GST_CAT_CAPS]		= "04;34",
-  [GST_CAT_CLOCK]		= "00;33",		/* !! */
-  [GST_CAT_ELEMENT_PADS]	= "01;37;41",		/* !! */
-  [GST_CAT_ELEMENT_FACTORY]	= "01;37;41",		/* !! */
-  [GST_CAT_PADS]		= "01;37;41",		/* !! */
-  [GST_CAT_PIPELINE]		= "01;37;41",		/* !! */
-  [GST_CAT_PLUGIN_LOADING]	= "00;36",
-  [GST_CAT_PLUGIN_ERRORS]	= "05;31",
-  [GST_CAT_PLUGIN_INFO]		= "00;36",
-  [GST_CAT_PROPERTIES]		= "00;37;44",		/* !! */
-  [GST_CAT_THREAD]		= "00;31",
-  [GST_CAT_TYPES]		= "01;37;41",		/* !! */
-  [GST_CAT_XML]			= "01;37;41",		/* !! */
-  [GST_CAT_NEGOTIATION]		= "07;34",
-  [GST_CAT_REFCOUNTING]		= "00;34:42",
-  [GST_CAT_EVENT]		= "01;37;41",		/* !! */
-  [GST_CAT_PARAMS]		= "00;30;43",		/* !! */
-
-  [GST_CAT_CALL_TRACE]		= "",
-  [31]				= "05;31",
+  /* [GST_CAT_GST_INIT]		= */ "07;37",
+  /* [GST_CAT_COTHREADS]	= */ "00;32",
+  /* [GST_CAT_COTHREAD_SWITCH]	= */ "00;37;42",
+  /* [GST_CAT_AUTOPLUG]		= */ "00;34",
+  /* [GST_CAT_AUTOPLUG_ATTEMPT]	= */ "00;36;44",
+  /* [GST_CAT_PARENTAGE]	= */ "01;37;41",	/* !! */
+  /* [GST_CAT_STATES]		= */ "00;31",
+  /* [GST_CAT_PLANNING]		= */ "07;35",
+  /* [GST_CAT_SCHEDULING]	= */ "00;35",
+  /* [GST_CAT_DATAFLOW]		= */ "00;32",
+  /* [GST_CAT_BUFFER]		= */ "00;32",
+  /* [GST_CAT_CAPS]		= */ "04;34",
+  /* [GST_CAT_CLOCK]		= */ "00;33",		/* !! */
+  /* [GST_CAT_ELEMENT_PADS]	= */ "01;37;41",	/* !! */
+  /* [GST_CAT_ELEMENT_FACTORY]	= */ "01;37;41",	/* !! */
+  /* [GST_CAT_PADS]		= */ "01;37;41",	/* !! */
+  /* [GST_CAT_PIPELINE]		= */ "01;37;41",	/* !! */
+  /* [GST_CAT_PLUGIN_LOADING]	= */ "00;36",
+  /* [GST_CAT_PLUGIN_ERRORS]	= */ "05;31",
+  /* [GST_CAT_PLUGIN_INFO]	= */ "00;36",
+  /* [GST_CAT_PROPERTIES]	= */ "00;37;44",	/* !! */
+  /* [GST_CAT_THREAD]		= */ "00;31",
+  /* [GST_CAT_TYPES]		= */ "01;37;41",	/* !! */
+  /* [GST_CAT_XML]		= */ "01;37;41",	/* !! */
+  /* [GST_CAT_NEGOTIATION]	= */ "07;34",
+  /* [GST_CAT_REFCOUNTING]	= */ "00;34;42",
+  /* [GST_CAT_EVENT]		= */ "01;37;41",	/* !! */
+  /* [GST_CAT_PARAMS]		= */ "00;30;43",	/* !! */
+  				     "",
+  				     "",
+  /* [GST_CAT_CALL_TRACE]	= */ "",
+  /* [31]			= */ "05;31",
 };
 
 /* colorization hash - DEPRACATED in favor of above */
@@ -153,7 +160,6 @@ inline gint _gst_debug_stringhash_color(gchar *file) {
 }
 
 
-
 /***** DEBUG system *****/
 GstDebugHandler _gst_debug_handler = gst_default_debug_handler;
 guint32 _gst_debug_categories = 0x00000000;
@@ -178,7 +184,7 @@ gst_default_debug_handler (gint category, gboolean incore,
 			   const gchar *file, const gchar *function,
 			   gint line, const gchar *debug_string,
 			   void *element, gchar *string)
-  __attribute__ ((no_instrument_function));
+  G_GNUC_NO_INSTRUMENT;
 
 void
 gst_default_debug_handler (gint category, gboolean incore,
@@ -188,37 +194,39 @@ gst_default_debug_handler (gint category, gboolean incore,
 {
   gchar *empty = "";
   gchar *elementname = empty,*location = empty;
-  int pthread_id = getpid();
-  int cothread_id = 0; /*FIXME*/
+  int pid = getpid();
+  int cothread_id = GPOINTER_TO_INT(g_static_private_get(&_gst_debug_cothread_index));
 #ifdef GST_DEBUG_COLOR
-  int pthread_color = pthread_id%6 + 31;
+  int pid_color = pid%6 + 31;
   int cothread_color = (cothread_id < 0) ? 37 : (cothread_id%6 + 31);
 #endif
 
   if (debug_string == NULL) debug_string = "";
 /*  if (category != GST_CAT_GST_INIT) */
-    location = g_strdup_printf("%s:%d%s:",function,line,debug_string);
+    location = g_strdup_printf ("%s(%d): %s: %s:",
+		                file, line, function, debug_string);
   if (element && GST_IS_ELEMENT (element))
 #ifdef GST_DEBUG_COLOR
-    elementname = g_strdup_printf (" \033[04m[%s]\033[00m", GST_OBJECT_NAME (element));
+    elementname = g_strdup_printf (" \033[04m[%s]\033[00m",
+		                   GST_OBJECT_NAME (element));
 #else
     elementname = g_strdup_printf (" [%s]", GST_OBJECT_NAME (element));
 #endif
 
 #ifdef GST_DEBUG_COLOR
-  fprintf(stderr,"DEBUG(\033[00;%dm%5d\033[00m:\033[00;%dm%2d\033[00m)\033["
-          "%s;%sm%s%s\033[00m %s\n",
-          pthread_color,pthread_id,cothread_color,cothread_id,incore?"00":"01",
-          _gst_category_colors[category],location,elementname,string);
+  fprintf (stderr, "DEBUG(\033[00;%dm%5d\033[00m:\033[00;%dm%2d\033[00m)\033["
+           "%s;%sm%s%s\033[00m %s\n",
+           pid_color, pid, cothread_color, cothread_id, incore ? "00" : "01",
+          _gst_category_colors[category], location, elementname, string);
 #else
   fprintf(stderr,"DEBUG(%5d:%2d)%s%s %s\n",
-          pthread_id,cothread_id,location,elementname,string);
+          pid, cothread_id, location, elementname, string);
 #endif /* GST_DEBUG_COLOR */
 
-  if (location != empty) g_free(location);
-  if (elementname != empty) g_free(elementname);
+  if (location != empty) g_free (location);
+  if (elementname != empty) g_free (elementname);
 
-  g_free(string);
+  g_free (string);
 }
 
 
@@ -269,14 +277,11 @@ gst_debug_disable_category (gint category) {
   _gst_debug_categories &= ~ (1 << category);
 }
 
-
-
-
 /***** INFO system *****/
 GstInfoHandler _gst_info_handler = gst_default_info_handler;
 guint32 _gst_info_categories = 0x00000001;
 
-
+/* FIXME:what does debug_string DO ??? */
 /**
  * gst_default_info_handler:
  * @category: category of the INFO message
@@ -290,6 +295,7 @@ guint32 _gst_info_categories = 0x00000001;
  *
  * Prints out the INFO mesage in a variant of the following form:
  *
+ *   FIXME: description should be fixed
  *   INFO:gst_function:542(args): [elementname] something neat happened
  */
 void
@@ -300,30 +306,32 @@ gst_default_info_handler (gint category, gboolean incore,
 {
   gchar *empty = "";
   gchar *elementname = empty,*location = empty;
-  int pthread_id = getpid();
-  int cothread_id = 0; /*FIXME*/
+  int pid = getpid();
+  int cothread_id = GPOINTER_TO_INT(g_static_private_get(&_gst_debug_cothread_index));
 #ifdef GST_DEBUG_COLOR
-  int pthread_color = pthread_id%6 + 31;
+  int pid_color = pid%6 + 31;
   int cothread_color = (cothread_id < 0) ? 37 : (cothread_id%6 + 31);
 #endif
 
   if (debug_string == NULL) debug_string = "";
   if (category != GST_CAT_GST_INIT)
-    location = g_strdup_printf("%s:%d%s:",function,line,debug_string);
+    location = g_strdup_printf ("%s(%d): %s: %s:",
+		                file, line, function, debug_string);
   if (element && GST_IS_ELEMENT (element))
-    elementname = g_strdup_printf (" \033[04m[%s]\033[00m", GST_OBJECT_NAME (element));
+    elementname = g_strdup_printf (" \033[04m[%s]\033[00m",
+		                   GST_OBJECT_NAME (element));
 
 /*
 #ifdef GST_DEBUG_ENABLED
 */
   #ifdef GST_DEBUG_COLOR
-    fprintf(stderr,"\033[01mINFO\033[00m (\033[00;%dm%5d\033[00m:\033[00;%dm%2d\033[00m)\033["
-            GST_DEBUG_CHAR_MODE ";%sm%s%s\033[00m %s\n",
-            pthread_color,pthread_id,cothread_color,cothread_id,
-            _gst_category_colors[category],location,elementname,string);
+    fprintf (stderr, "\033[01mINFO\033[00m (\033[00;%dm%5d\033[00m:\033[00;%dm%2d\033[00m)\033["
+             GST_DEBUG_CHAR_MODE ";%sm%s%s\033[00m %s\n",
+             pid_color, pid, cothread_color, cothread_id,
+             _gst_category_colors[category], location, elementname, string);
   #else
-    fprintf(stderr,"INFO (%5d:%2d)%s%s %s\n",
-            pthread_id,cothread_id,location,elementname,string);
+    fprintf (stderr, "INFO (%5d:%2d)%s%s %s\n",
+            pid, cothread_id, location, elementname, string);
 #endif /* GST_DEBUG_COLOR */
 /*
 #else
@@ -338,10 +346,10 @@ gst_default_info_handler (gint category, gboolean incore,
 #endif
 */
 
-  if (location != empty) g_free(location);
-  if (elementname != empty) g_free(elementname);
+  if (location != empty) g_free (location);
+  if (elementname != empty) g_free (elementname);
 
-  g_free(string);
+  g_free (string);
 }
 
 /**
@@ -478,11 +486,13 @@ gst_default_error_handler (gchar *file, gchar *function,
 
 
 /***** DEBUG system *****/
+#ifdef GST_DEBUG_ENABLED
+#ifdef GST_DEBUG_ENABLED
 GHashTable *__gst_function_pointers = NULL;
 /* FIXME make this thread specific */
 /* static GSList *stack_trace = NULL; */
 
-gchar *_gst_debug_nameof_funcptr (void *ptr) __attribute__ ((no_instrument_function));
+gchar *_gst_debug_nameof_funcptr (void *ptr) G_GNUC_NO_INSTRUMENT;
 
 gchar *
 _gst_debug_nameof_funcptr (void *ptr)
@@ -497,11 +507,13 @@ _gst_debug_nameof_funcptr (void *ptr)
     return g_strdup_printf("%p",ptr);
   }
   return NULL;
+#endif
 }
+#endif
 
 
 #ifdef GST_ENABLE_FUNC_INSTRUMENTATION
-void __cyg_profile_func_enter(void *this_fn,void *call_site) __attribute__ ((no_instrument_function));
+void __cyg_profile_func_enter(void *this_fn,void *call_site) G_GNUC_NO_INSTRUMENT;
 void __cyg_profile_func_enter(void *this_fn,void *call_site) 
 {
   gchar *name = _gst_debug_nameof_funcptr (this_fn);
@@ -514,7 +526,7 @@ void __cyg_profile_func_enter(void *this_fn,void *call_site)
   g_free (site);
 }
 
-void __cyg_profile_func_exit(void *this_fn,void *call_site) __attribute__ ((no_instrument_function));
+void __cyg_profile_func_exit(void *this_fn,void *call_site) G_GNUC_NO_INSTRUMENT;
 void __cyg_profile_func_exit(void *this_fn,void *call_site) 
 {
   gchar *name = _gst_debug_nameof_funcptr (this_fn);
@@ -551,3 +563,14 @@ gst_debug_print_stack_trace (void)
 }
 
 #endif /* GST_ENABLE_FUNC_INTSTRUMENTATION */
+
+void *
+_gst_debug_register_funcptr (void *ptr, gchar *ptrname)
+{
+  if (!__gst_function_pointers)
+    __gst_function_pointers = g_hash_table_new (g_direct_hash, g_direct_equal);
+  if (!g_hash_table_lookup (__gst_function_pointers, ptr))
+    g_hash_table_insert (__gst_function_pointers, ptr, ptrname);
+  return ptr;
+}
+
