@@ -34,6 +34,7 @@
 #include <gst/gsttype.h>
 #include <gst/gstscheduler.h>
 #include <gst/gstautoplug.h>
+#include <gst/gsturi.h>
 
 #include "gstxmlregistry.h"
 
@@ -612,7 +613,7 @@ gst_xml_registry_load (GstRegistry *registry)
   seconds = g_timer_elapsed (timer, NULL);
   g_timer_destroy (timer);
 
-  g_print ("registry: loaded %s in %f seconds\n          (%s)\n", 
+  GST_INFO (0, "registry: loaded %s in %f seconds\n          (%s)", 
 	   registry->name, seconds, xmlregistry->location);
 
   CLASS (xmlregistry)->close_func (xmlregistry);
@@ -624,8 +625,13 @@ gst_xml_registry_load (GstRegistry *registry)
 static GstRegistryReturn 
 gst_xml_registry_load_plugin (GstRegistry *registry, GstPlugin *plugin)
 {
+  GError *error = NULL;
+
   /* FIXME: add gerror support */
-  if (!gst_plugin_load_plugin (plugin, NULL)) {
+  if (!gst_plugin_load_plugin (plugin, &error)) {
+    if (error) {
+      g_warning ("could not load plugin %s: %s", plugin->name, error->message);
+    }
     return GST_REGISTRY_PLUGIN_LOAD_ERROR;
   }
 
@@ -755,10 +761,10 @@ gst_xml_registry_parse_autoplug_factory (GMarkupParseContext *context, const gch
 }
 
 static gboolean
-gst_xml_registry_parse_cache_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
+gst_xml_registry_parse_index_factory (GMarkupParseContext *context, const gchar *tag, const gchar *text,
                                       gsize text_len, GstXMLRegistry *registry, GError **error)
 {
-  GstCacheFactory *factory = GST_CACHE_FACTORY (registry->current_feature);
+  GstIndexFactory *factory = GST_INDEX_FACTORY (registry->current_feature);
 
   if (!strcmp (tag, "name")) {
     registry->current_feature->name = g_strndup (text, text_len);
@@ -769,6 +775,30 @@ gst_xml_registry_parse_cache_factory (GMarkupParseContext *context, const gchar
   return TRUE;
 }
 
+static gboolean
+gst_xml_registry_parse_uri_handler (GMarkupParseContext *context, const gchar *tag, const gchar *text,
+                                    gsize text_len, GstXMLRegistry *registry, GError **error)
+{
+  GstURIHandler *handler = GST_URI_HANDLER (registry->current_feature);
+
+  if (!strcmp (tag, "name")) {
+    registry->current_feature->name = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "uri")) {
+    handler->uri = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "longdesc")) {
+    handler->longdesc = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "element")) {
+    handler->element = g_strndup (text, text_len);
+  }
+  else if (!strcmp (tag, "property")) {
+    handler->property = g_strndup (text, text_len);
+  }
+  return TRUE;
+}
+
 static gboolean
 gst_xml_registry_parse_padtemplate (GMarkupParseContext *context, const gchar *tag, const gchar *text,
                                     gsize text_len, GstXMLRegistry *registry, GError **error)
@@ -865,6 +895,7 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
 	}
 	if (feature) {
 	  xmlregistry->current_feature = feature;
+
 	  if (GST_IS_ELEMENT_FACTORY (feature)) {
 	    GstElementFactory *factory = GST_ELEMENT_FACTORY (feature);
 
@@ -873,17 +904,24 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
 	    factory->padtemplates = NULL;
 	    factory->rank = 0;
 	    xmlregistry->parser = gst_xml_registry_parse_element_factory;
+	    break;
 	  }
-	  else if (GST_IS_TYPE_FACTORY (feature))
+	  else if (GST_IS_TYPE_FACTORY (feature)) {
 	    xmlregistry->parser = gst_xml_registry_parse_type_factory;
+	  }
 	  else if (GST_IS_SCHEDULER_FACTORY (feature)) {
 	    xmlregistry->parser = gst_xml_registry_parse_scheduler_factory;
             GST_SCHEDULER_FACTORY (feature)->type = 0;
 	  }
-	  else if (GST_IS_AUTOPLUG_FACTORY (feature))
+	  else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
 	    xmlregistry->parser = gst_xml_registry_parse_autoplug_factory;
-	  else if (GST_IS_CACHE_FACTORY (feature))
-	    xmlregistry->parser = gst_xml_registry_parse_cache_factory;
+	  }
+	  else if (GST_IS_INDEX_FACTORY (feature)) {
+	    xmlregistry->parser = gst_xml_registry_parse_index_factory;
+	  }
+	  else if (GST_IS_URI_HANDLER (feature)) {
+	    xmlregistry->parser = gst_xml_registry_parse_uri_handler;
+	  }
 	  else {
             g_warning ("unkown feature type");
 	  }
@@ -922,11 +960,17 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
     case GST_XML_REGISTRY_PROPERTIES:
     {
       gint name_index;
-      
+      GstPropsEntry *entry = NULL;
+
       name_index = find_index_for ("name", attribute_names);
       if (name_index < 0)
 	break;
 
+      if (!strncmp (element_name, "list", 4)) {
+	xmlregistry->in_list = TRUE;
+	xmlregistry->list_name = g_strdup (attribute_values[name_index]);
+      }
+
       if (!strncmp (element_name, "int", 3)) {
 	gint value;
         gint index;
@@ -934,9 +978,9 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
 	if ((index = find_index_for ("value", attribute_names)) < 0) 
           break;
 	sscanf (attribute_values[index], "%d", &value);
-	gst_props_add_entry (xmlregistry->props, 
-			     gst_props_entry_new (attribute_values[name_index], 
-			                          GST_PROPS_INT (value)));
+
+	entry = gst_props_entry_new (attribute_values[name_index], 
+		                     GST_PROPS_INT (value));
       }
       else if (!strncmp (element_name, "range", 5)) {
 	gint min, max;
@@ -948,8 +992,8 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
           break;
 	sscanf (attribute_values[min_idx], "%d", &min);
 	sscanf (attribute_values[max_idx], "%d", &max);
-	gst_props_add_entry (xmlregistry->props, 
-			gst_props_entry_new (attribute_values[name_index], GST_PROPS_INT_RANGE (min, max)));
+
+	entry = gst_props_entry_new (attribute_values[name_index], GST_PROPS_INT_RANGE (min, max));
       }
       else if (!strncmp (element_name, "float", 5)) {
 	gfloat value;
@@ -958,8 +1002,8 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
 	if ((index = find_index_for ("value", attribute_names)) < 0) 
           break;
 	sscanf (attribute_values[index], "%f", &value);
-	gst_props_add_entry (xmlregistry->props, 
-			gst_props_entry_new (attribute_values[name_index], GST_PROPS_FLOAT (value)));
+	
+	entry = gst_props_entry_new (attribute_values[name_index], GST_PROPS_FLOAT (value));
       }
       else if (!strncmp (element_name, "floatrange", 10)) {
 	gfloat min, max;
@@ -971,8 +1015,8 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
           break;
 	sscanf (attribute_values[min_idx], "%f", &min);
 	sscanf (attribute_values[max_idx], "%f", &max);
-	gst_props_add_entry (xmlregistry->props, 
-			gst_props_entry_new (attribute_values[name_index], GST_PROPS_FLOAT_RANGE (min, max)));
+
+	entry = gst_props_entry_new (attribute_values[name_index], GST_PROPS_FLOAT_RANGE (min, max));
       }
       else if (!strncmp (element_name, "boolean", 7)) {
 	gboolean value = TRUE;
@@ -982,8 +1026,8 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
           break;
 	if (!strcmp (attribute_values[index], "false")) 
           value = FALSE;
-	gst_props_add_entry (xmlregistry->props, 
-			gst_props_entry_new (attribute_values[name_index], GST_PROPS_BOOLEAN (value)));
+
+	entry = gst_props_entry_new (attribute_values[name_index], GST_PROPS_BOOLEAN (value));
       }
       else if (!strncmp (element_name, "fourcc", 6)) {
 	guint32 value;
@@ -992,17 +1036,24 @@ gst_xml_registry_start_element (GMarkupParseContext *context,
 	if ((index = find_index_for ("hexvalue", attribute_names)) < 0) 
           break;
 	sscanf (attribute_values[index], "%08x", &value);
-	gst_props_add_entry (xmlregistry->props, 
-			gst_props_entry_new (attribute_values[name_index], GST_PROPS_FOURCC (value)));
+
+	entry = gst_props_entry_new (attribute_values[name_index], GST_PROPS_FOURCC (value));
       }
       else if (!strncmp (element_name, "string", 6)) {
         gint index;
 
 	if ((index = find_index_for ("value", attribute_names)) < 0) 
           break;
-	gst_props_add_entry (xmlregistry->props, 
-			gst_props_entry_new (attribute_values[name_index], 
-					      GST_PROPS_STRING (attribute_values[index])));
+
+        entry = gst_props_entry_new (attribute_values[name_index], 
+				     GST_PROPS_STRING (attribute_values[index]));
+      }
+      /* add property to list or parent */
+      if (entry) {
+        if (xmlregistry->in_list)
+          xmlregistry->entry_list = g_list_prepend (xmlregistry->entry_list, entry);
+        else
+          gst_props_add_entry (xmlregistry->props, entry);
       }
       break;
     }
@@ -1093,7 +1144,23 @@ gst_xml_registry_end_element (GMarkupParseContext *context,
       }
       break;
     case GST_XML_REGISTRY_PROPERTIES:
-      if (!strcmp (element_name, "properties")) {
+      if (!strncmp (element_name, "list", 4)) {
+	GstPropsEntry *entry;
+	
+	xmlregistry->entry_list = g_list_reverse (xmlregistry->entry_list);
+	
+        entry = gst_props_entry_new (xmlregistry->list_name, 
+			             GST_PROPS_GLIST (xmlregistry->entry_list));
+
+        gst_props_add_entry (xmlregistry->props, entry);
+	g_list_free (xmlregistry->entry_list);
+	g_free (xmlregistry->list_name);
+
+	xmlregistry->entry_list = NULL;
+	xmlregistry->list_name = NULL;
+	xmlregistry->in_list = FALSE;
+      }
+      else if (!strcmp (element_name, "properties")) {
 	xmlregistry->state = GST_XML_REGISTRY_CAPSCOMP;
 	xmlregistry->parser = NULL;
       }
@@ -1408,8 +1475,16 @@ gst_xml_registry_save_feature (GstXMLRegistry *xmlregistry, GstPluginFeature *fe
   else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
     PUT_ESCAPED ("longdesc", GST_AUTOPLUG_FACTORY (feature)->longdesc);
   }
-  else if (GST_IS_CACHE_FACTORY (feature)) {
-    PUT_ESCAPED ("longdesc", GST_CACHE_FACTORY (feature)->longdesc);
+  else if (GST_IS_INDEX_FACTORY (feature)) {
+    PUT_ESCAPED ("longdesc", GST_INDEX_FACTORY (feature)->longdesc);
+  }
+  else if (GST_IS_URI_HANDLER (feature)) {
+    GstURIHandler *handler = GST_URI_HANDLER (feature);
+
+    PUT_ESCAPED ("uri", handler->uri);
+    PUT_ESCAPED ("longdesc", handler->longdesc);
+    PUT_ESCAPED ("element", handler->element);
+    PUT_ESCAPED ("property", handler->property);
   }
   return TRUE;
 }
